/*
 * Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <unistd.h>
#include "mpm_transport.h"

#define TEST_BASE_HYP1 (0x0c000000)
#define TEST_LENGTH_HYP1 (0x1000)
#define TEST_BASE_HYP2 (0x0c008000)
#define TEST_LENGTH_HYP2 (0x2000)

#define TEST_DESTINATION1 0x822000000
#define TEST_SOURCE1 0x839000000
#define TEST_BUFFER_SIZE 0x400000

int main()
{
	int i;
	mpm_transport_h h;
	uint32_t *bufp, *bufp2;
	union char_int {
		char buf[4];
		uint32_t num;
	} val;
	int result = 0;

	mpm_transport_open_t ocfg = {
		.open_mode	= (O_SYNC|O_RDWR),
		.msec_timeout = 5000,
	};

	mpm_transport_mmap_t mcfg = {
		.mmap_prot	= (PROT_READ|PROT_WRITE),
		.mmap_flags	= MAP_SHARED,
	};

	/* Write to memory */
	printf("testing hyperlink mmap write\n");
	h = mpm_transport_open("arm-remote-hyplnk-0-no-dma", &ocfg);
	if (!h) {
		printf("open failed\n");
		return -1;
	}
	
	bufp = (uint32_t *) mpm_transport_mmap(h, TEST_BASE_HYP2, TEST_LENGTH_HYP2, &mcfg);
	if (!bufp) {
		printf("mmap failed\n");
		return -1;
	}
	for (i = 0; i < TEST_LENGTH_HYP2; i += 4) {
		bufp[i/4] = (uint32_t) &(bufp[i/4]);
	}

	bufp2 = (uint32_t *) mpm_transport_mmap(h, TEST_BASE_HYP1, TEST_LENGTH_HYP1, &mcfg);
	if (!bufp2) {
		printf("mmap failed\n");
		return -1;
	}
	for (i = 0; i < TEST_LENGTH_HYP1; i += 4) {
		bufp2[i/4] = i;
	}

	sleep(1);
	mpm_transport_close(h);
	printf("completed hyperlink mmap read and check\n");

	/* testing read/write */
	printf("testing hyperlink mpm_transport_write and mpm_transport_read\n");
	h = mpm_transport_open("arm-remote-hyplnk-0-no-dma", &ocfg);

	for (i = 0; i < TEST_LENGTH_HYP1; i += 4) {
		val.num = TEST_BASE_HYP1 - i;
		if (mpm_transport_write(h, TEST_BASE_HYP1 + i, 4, val.buf, NULL)) {
			printf("write failed at address 0x%x\n", TEST_BASE_HYP1 + i);
			result = -1;
			break;
		}
	}
	for (i = 0; i < TEST_LENGTH_HYP1; i += 4) {
		if (mpm_transport_read(h, TEST_BASE_HYP1 + i, 4, val.buf, NULL)) {
			printf("read failed at address 0x%x\n", TEST_BASE_HYP1 + i);
			result = -1;
			break;
		}
		if (val.num != TEST_BASE_HYP1 - i) {
			printf("read/write did not match at address 0x%x, val.num 0x%x\n",
				TEST_BASE_HYP1 + i, val.num);
			result = -1;
			break;
		}
	}
	for (i = 0; i < TEST_LENGTH_HYP2; i += 4) {
		val.num = TEST_BASE_HYP2 + i;
		if (mpm_transport_write(h, TEST_BASE_HYP2 + i, 4, val.buf, NULL)) {
			printf("write failed at address 0x%x\n", TEST_BASE_HYP2 + i);
			result = -1;
			break;
		}
	}
	for (i = 0; i < TEST_LENGTH_HYP2; i += 4) {
		if (mpm_transport_read(h, TEST_BASE_HYP2 + i, 4, val.buf, NULL)) {
			printf("read failed at address 0x%x\n", TEST_BASE_HYP2 + i);
			result = -1;
			break;
		}
		if (val.num != TEST_BASE_HYP2 + i) {
			printf("read/write did not match at address 0x%x, val.num 0x%x\n",
				TEST_BASE_HYP2 + i, val.num);
			result = -1;
			break;
		}
	}

	sleep(1);
	mpm_transport_close(h);

	printf("Testing remote 36-bit put\n");
	h = mpm_transport_open("arm-remote-hyplnk64-0",&ocfg);
	if (!h) {
		printf("open failed for arm-remote-hyplnk64-0\n");
		return -1;
	}
	mpm_transport_put_initiate64(h, TEST_DESTINATION1, TEST_SOURCE1, TEST_BUFFER_SIZE, true, NULL);
	sleep(2);
	mpm_transport_close(h);

	if (result) {
		printf("mpm transport: hyperlink remote test failed\n");
	} else {
		printf("mpm transport: hyperlink remote test passed\n");
	}
	return 0;
}

